package de.fzj.unicore.xuudb.server;

import java.io.File;
import java.util.List;
import java.util.Properties;

import junit.framework.TestCase;
import de.fzJuelich.unicore.xuudb.LoginDataType;
import de.fzj.unicore.xuudb.Configuration;
import de.fzj.unicore.xuudb.X509Utils;
import de.fzj.unicore.xuudb.XUUDBInit;
import de.fzj.unicore.xuudb.server.db.StorageFactory;
import eu.unicore.bugsreporter.annotation.FunctionalTest;
import eu.unicore.bugsreporter.annotation.FunctionalTests;

public class TestXUUDB extends TestCase {
	
	static XUUDB xuudb;
	private static String file;
	private static boolean haveInit=false;
	
	@Override
	protected void setUp()throws Exception{
		if(haveInit){
			LoginDataType removeRequest=LoginDataType.Factory.newInstance();
			xuudb.remove(removeRequest);
			return;
		}
		haveInit=true;
		Properties prop=new Properties();
		prop.setProperty(Configuration.XUUDBTYPE, getMode());
		new XUUDBInit(prop);
		xuudb=new XUUDB();
		file=System.getProperty("java.io.tmpdir")+File.separator+"xuudb_test_"+System.currentTimeMillis();
		xuudb.setDatabase(StorageFactory.getDatabase(file));
	}
	
	protected String getMode(){
		return "normal";
	}
	
	protected boolean isDNMode(){
		return "dn".equals(getMode());
	}	
	
	@FunctionalTests({@FunctionalTest(id="xuudb_remove"), 
			@FunctionalTest(id="xuudb_add"),
			@FunctionalTest(id="xuudb_list"),
			@FunctionalTest(id="xuudb_update")})
	public void testAddRemoveIndividualXlogins()throws Exception{
		
		LoginDataType addRequest=getAddRequest("xlogin1");
		xuudb.add(addRequest);
		
		assertEquals(1,getEntries().length);

		//add a second xlogin
		addRequest=getAddRequest("xlogin2");
		xuudb.add(addRequest);
		
		LoginDataType[]os=getEntries();
		assertEquals(1,os.length);
		LoginDataType x=os[0];
		assertEquals("xlogin1:xlogin2",x.getXlogin());
		
		//add a third xlogin
		addRequest=getAddRequest("xlogin3");
		xuudb.add(addRequest);
		
		os=getEntries();
		assertEquals(1,os.length);
		x=os[0];
		assertEquals("xlogin1:xlogin2:xlogin3",x.getXlogin());
		
		//remove one
		LoginDataType removeRequest=getRemoveRequest("xlogin2");
		xuudb.remove(removeRequest);
		os=getEntries();
		assertEquals(1,os.length);
		x=os[0];
		assertEquals("xlogin1:xlogin3",x.getXlogin());
		//remove 2nd one
		removeRequest=getRemoveRequest("xlogin3");
		xuudb.remove(removeRequest);
		os=getEntries();
		assertEquals(1,os.length);
		x=os[0];
		assertEquals("xlogin1",x.getXlogin());
		
		//remove last one
		removeRequest=getRemoveRequest("xlogin1");
		xuudb.remove(removeRequest);
		os=getEntries();
		assertEquals(0,os.length);
		
	}

	public void testDoNotAddSameXlogin()throws Exception{
		
		LoginDataType addRequest=getAddRequest("xlogin1");
		xuudb.add(addRequest);
		
		assertEquals(1,getEntries().length);

		Xlogin x=new Xlogin(getEntries()[0].getXlogin());
		assertEquals(1, x.getNumberOfLogins());

		addRequest=getAddRequest("xlogin2");
		xuudb.add(addRequest);

		x=new Xlogin(getEntries()[0].getXlogin());
		assertEquals(2, x.getNumberOfLogins());
		
		//add the same xlogin
		addRequest=getAddRequest("xlogin1");
		xuudb.add(addRequest);

		x=new Xlogin(getEntries()[0].getXlogin());
		assertEquals(2, x.getNumberOfLogins());
		
	}

	public void testRemoveFullXlogin()throws Exception{
		Properties prop=new Properties();
		new XUUDBInit(prop);
		xuudb=new XUUDB();
		xuudb.setDatabase(StorageFactory.getDatabase(System.getProperty("java.io.tmpdir")+File.separator+"xuudb_test_"+System.currentTimeMillis()));
		
		LoginDataType addRequest=getAddRequest("xlogin1:xlogin2:xlogin3");
		xuudb.add(addRequest);
		assertEquals(1,getEntries().length);
		
		//remove last one
		LoginDataType removeRequest=getRemoveRequest("xlogin1:xlogin2:xlogin3");
		xuudb.remove(removeRequest);
		LoginDataType[]os=getEntries();
		assertEquals(0,os.length);
		
	}

	public void testRemoveByDN()throws Exception{
		Properties prop=new Properties();
		new XUUDBInit(prop);
		xuudb=new XUUDB();
		xuudb.setDatabase(StorageFactory.getDatabase(System.getProperty("java.io.tmpdir")+File.separator+"xuudb_test_"+System.currentTimeMillis()));
		
		LoginDataType addRequest=getAddRequest("xlogin1:xlogin2:xlogin3");
		xuudb.add(addRequest);
		assertEquals(1,getEntries().length);
		
		//remove last one
		LoginDataType removeRequest=getRemoveByDNRequest();
		xuudb.remove(removeRequest);
		LoginDataType[]os=getEntries();
		assertEquals(0,os.length);
		
	}

	
	public void testBasicCRUD()throws Exception{
		LoginDataType addRequest=getAddRequest("xlogin1");
		xuudb.add(addRequest);
		assertEquals(1,getEntries().length);
		
		LoginDataType queryResponse=doQuery();
		assertEquals("xlogin1",queryResponse.getXlogin());
		
		List<String>gcids=xuudb.getGCIDs();
		assertNotNull(gcids);
		assertEquals(1,gcids.size());
		assertEquals("foo",gcids.get(0));
		
		
		LoginDataType removeRequest=getRemoveByXloginRequest("xlogin1");
		xuudb.remove(removeRequest);
		assertEquals(0,getEntries().length);
		
		queryResponse=doQuery();
		assertNull(queryResponse.getXlogin());
		
	
	}
	
	private LoginDataType[] getEntries()throws Exception{
		LoginDataType query=LoginDataType.Factory.newInstance();
		query.setGcID("foo");
		return xuudb.listDB(query);
	}
	
	protected LoginDataType getAddRequest(String login)throws Exception{
		String cert=X509Utils.getStringFromPEMFile(new File("src/test/resources/demo-user.pem"));
		LoginDataType add=LoginDataType.Factory.newInstance();
		add.setGcID("foo");
		add.setRole("user");
		add.setXlogin(login);
		add.setToken(cert);
		return add;
	}

	private LoginDataType doQuery()throws Exception{
		String cert=X509Utils.getStringFromPEMFile(new File("src/test/resources/demo-user.pem"));
		return xuudb.checkCert("foo", cert);
	}

	private LoginDataType getRemoveByXloginRequest(String login)throws Exception{
		LoginDataType add=LoginDataType.Factory.newInstance();
		add.setXlogin(login);
		return add;
	}
	

	private LoginDataType getRemoveRequest(String login)throws Exception{
		String cert=X509Utils.getStringFromPEMFile(new File("src/test/resources/demo-user.pem"));
		LoginDataType add=LoginDataType.Factory.newInstance();
		add.setGcID("foo");
		add.setXlogin(login);
		add.setToken(cert);
		return add;
	}
	
	private LoginDataType getRemoveByDNRequest()throws Exception{
		LoginDataType add=LoginDataType.Factory.newInstance();
		String cert=X509Utils.getStringFromPEMFile(new File("src/test/resources/demo-user.pem"));
		String dn=SecurityToken.getDN(cert, false);
		add.setToken(dn);
		return add;
	}
	
	
}
